import {
  Contract,
  ContractType,
  NetworkId,
  SendGovernanceInstruction,
  SyncOp,
  WormholeAddress,
  WormholeNetwork,
} from "./Contract";
import { ethers } from "ethers";
import { ChainName } from "../chains";
import { SetValidPeriod } from "../governance_payload";

export class EvmPythUpgradable implements Contract<EvmPythUpgradableState> {
  public type = ContractType.EvmPythUpgradable;
  public networkId;
  private address;

  private contract: ethers.Contract;

  constructor(
    networkId: NetworkId,
    address: string,
    contract: ethers.Contract
  ) {
    this.networkId = networkId;
    this.address = address;
    this.contract = contract;
  }

  public getAddress() {
    return this.address;
  }

  // TODO: these getters will need the full PythUpgradable ABI
  public async getAuthority(): Promise<WormholeAddress> {
    // FIXME: read from data sources
    return {
      emitter: "123454",
      chainId: 1,
      network: "mainnet",
    };
  }

  // get the chainId that identifies this contract
  public async getChain(): Promise<ChainName> {
    // FIXME: read from data sources
    return "polygon";
  }

  public async getState(): Promise<EvmPythUpgradableState> {
    const bytecodeSha = ethers.utils.sha256(
      (await this.contract.provider.getCode(this.contract.address)) as string
    );
    const validTimePeriod =
      (await this.contract.getValidTimePeriod()) as bigint;
    return {
      bytecodeSha,
      validTimePeriod: validTimePeriod.toString(),
    };
  }

  public async sync(target: EvmPythUpgradableState): Promise<SyncOp[]> {
    const myState = await this.getState();
    const authority = await this.getAuthority();
    const myChainId = await this.getChain();
    const whInstructions = [];

    if (myState.validTimePeriod !== target.validTimePeriod) {
      whInstructions.push(
        new SetValidPeriod(myChainId, BigInt(target.validTimePeriod))
      );
    }

    return whInstructions.map(
      (value) =>
        new SendGovernanceInstruction(
          value,
          authority,
          this.submitGovernanceVaa
        )
    );
  }

  public async submitGovernanceVaa(vaa: string): Promise<boolean> {
    // FIXME: also needs the full PythUpgradable ABI
    // await this.contract.executeGovernanceInstruction("0x" + vaa)
    return true;
  }
}

export interface EvmPythUpgradableState {
  bytecodeSha: string;
  // bigint serialized as a string
  validTimePeriod: string;
}
